@@ -1,23 +1,23 @@
-""" Multicast DNS Service Discovery for Python, v0.14-wmcbrine
-    Copyright 2003 Paul Scott-Murphy, 2014 William McBrine
+"""Multicast DNS Service Discovery for Python, v0.14-wmcbrine
+Copyright 2003 Paul Scott-Murphy, 2014 William McBrine
 
-    This module provides a framework for the use of DNS Service Discovery
-    using IP multicast.
+This module provides a framework for the use of DNS Service Discovery
+using IP multicast.
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-    Lesser General Public License for more details.
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
-    USA
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
+USA
 """
 
 import asyncio
@@ -39,7 +39,11 @@ from ._logger import QuietLogger, log
 from ._protocol.outgoing import DNSOutgoing
 from ._services import ServiceListener
 from ._services.browser import ServiceBrowser
-from ._services.info import ServiceInfo, instance_name_from_service_info
+from ._services.info import (
+    AsyncServiceInfo,
+    ServiceInfo,
+    instance_name_from_service_info,
+)
 from ._services.registry import ServiceRegistry
 from ._transport import _WrappedTransport
 from ._updates import RecordUpdateListener
@@ -117,7 +121,7 @@ def async_send_with_transport(
             return
     if log_debug:
         log.debug(
-            'Sending to (%s, %d) via [socket %s (%s)] (%d bytes #%d) %r as %r...',
+            "Sending to (%s, %d) via [socket %s (%s)] (%d bytes #%d) %r as %r...",
             real_addr,
             port or _MDNS_PORT,
             transport.fileno,
@@ -136,7 +140,6 @@ def async_send_with_transport(
 
 
 class Zeroconf(QuietLogger):
-
     """Implementation of Zeroconf Multicast DNS Service Discovery
 
     Supports registration, unregistration, queries and browsing.
@@ -169,12 +172,12 @@ class Zeroconf(QuietLogger):
 
         self.done = False
 
-        if apple_p2p and sys.platform != 'darwin':
-            raise RuntimeError('Option `apple_p2p` is not supported on non-Apple platforms.')
+        if apple_p2p and sys.platform != "darwin":
+            raise RuntimeError("Option `apple_p2p` is not supported on non-Apple platforms.")
 
         self.unicast = unicast
         listen_socket, respond_sockets = create_sockets(interfaces, unicast, ip_version, apple_p2p=apple_p2p)
-        log.debug('Listen socket %s, respond sockets %s', listen_socket, respond_sockets)
+        log.debug("Listen socket %s, respond sockets %s", listen_socket, respond_sockets)
 
         self.engine = AsyncEngine(self, listen_socket, respond_sockets)
 
@@ -257,11 +260,21 @@ class Zeroconf(QuietLogger):
             _resolve_all_futures_to_none(notify_futures)
 
     def get_service_info(
-        self, type_: str, name: str, timeout: int = 3000, question_type: Optional[DNSQuestionType] = None
+        self,
+        type_: str,
+        name: str,
+        timeout: int = 3000,
+        question_type: Optional[DNSQuestionType] = None,
     ) -> Optional[ServiceInfo]:
         """Returns network's service information for a particular
         name and type, or None if no service matches by the timeout,
-        which defaults to 3 seconds."""
+        which defaults to 3 seconds.
+
+        :param type_: fully qualified service type name
+        :param name: the name of the service
+        :param timeout: milliseconds to wait for a response
+        :param question_type: The type of questions to ask (DNSQuestionType.QM or DNSQuestionType.QU)
+        """
         info = ServiceInfo(type_, name)
         if info.request(self, timeout, question_type):
             return info
@@ -350,7 +363,9 @@ class Zeroconf(QuietLogger):
         """
         assert self.loop is not None
         run_coro_with_timeout(
-            await_awaitable(self.async_update_service(info)), self.loop, _REGISTER_TIME * _REGISTER_BROADCASTS
+            await_awaitable(self.async_update_service(info)),
+            self.loop,
+            _REGISTER_TIME * _REGISTER_BROADCASTS,
         )
 
     async def async_update_service(self, info: ServiceInfo) -> Awaitable:
@@ -360,6 +375,27 @@ class Zeroconf(QuietLogger):
         self.registry.async_update(info)
         return asyncio.ensure_future(self._async_broadcast_service(info, _REGISTER_TIME, None))
 
+    async def async_get_service_info(
+        self,
+        type_: str,
+        name: str,
+        timeout: int = 3000,
+        question_type: Optional[DNSQuestionType] = None,
+    ) -> Optional[AsyncServiceInfo]:
+        """Returns network's service information for a particular
+        name and type, or None if no service matches by the timeout,
+        which defaults to 3 seconds.
+
+        :param type_: fully qualified service type name
+        :param name: the name of the service
+        :param timeout: milliseconds to wait for a response
+        :param question_type: The type of questions to ask (DNSQuestionType.QM or DNSQuestionType.QU)
+        """
+        info = AsyncServiceInfo(type_, name)
+        if await info.async_request(self, timeout, question_type):
+            return info
+        return None
+
     async def _async_broadcast_service(
         self,
         info: ServiceInfo,
@@ -426,7 +462,9 @@ class Zeroconf(QuietLogger):
         """
         assert self.loop is not None
         run_coro_with_timeout(
-            self.async_unregister_service(info), self.loop, _UNREGISTER_TIME * _REGISTER_BROADCASTS
+            self.async_unregister_service(info),
+            self.loop,
+            _UNREGISTER_TIME * _REGISTER_BROADCASTS,
         )
 
     async def async_unregister_service(self, info: ServiceInfo) -> Awaitable:
@@ -479,7 +517,9 @@ class Zeroconf(QuietLogger):
         """
         assert self.loop is not None
         run_coro_with_timeout(
-            self.async_unregister_all_services(), self.loop, _UNREGISTER_TIME * _REGISTER_BROADCASTS
+            self.async_unregister_all_services(),
+            self.loop,
+            _UNREGISTER_TIME * _REGISTER_BROADCASTS,
         )
 
     async def async_check_service(
@@ -504,7 +544,7 @@ class Zeroconf(QuietLogger):
                     raise NonUniqueNameException
 
                 # change the name and look for a conflict
-                info.name = f'{instance_name}-{next_instance_number}.{info.type}'
+                info.name = f"{instance_name}-{next_instance_number}.{info.type}"
                 next_instance_number += 1
                 service_type_name(info.name, strict=strict)
                 next_time = now
@@ -520,7 +560,9 @@ class Zeroconf(QuietLogger):
             next_time += _CHECK_TIME
 
     def add_listener(
-        self, listener: RecordUpdateListener, question: Optional[Union[DNSQuestion, List[DNSQuestion]]]
+        self,
+        listener: RecordUpdateListener,
+        question: Optional[Union[DNSQuestion, List[DNSQuestion]]],
     ) -> None:
         """Adds a listener for a given question.  The listener will have
         its update_record method called when information is available to
@@ -540,7 +582,9 @@ class Zeroconf(QuietLogger):
         self.loop.call_soon_threadsafe(self.record_manager.async_remove_listener, listener)
 
     def async_add_listener(
-        self, listener: RecordUpdateListener, question: Optional[Union[DNSQuestion, List[DNSQuestion]]]
+        self,
+        listener: RecordUpdateListener,
+        question: Optional[Union[DNSQuestion, List[DNSQuestion]]],
     ) -> None:
         """Adds a listener for a given question.  The listener will have
         its update_record method called when information is available to
@@ -588,11 +632,23 @@ class Zeroconf(QuietLogger):
 
         for packet_num, packet in enumerate(out.packets()):
             if len(packet) > _MAX_MSG_ABSOLUTE:
-                self.log_warning_once("Dropping %r over-sized packet (%d bytes) %r", out, len(packet), packet)
+                self.log_warning_once(
+                    "Dropping %r over-sized packet (%d bytes) %r",
+                    out,
+                    len(packet),
+                    packet,
+                )
                 return
             for send_transport in transports:
                 async_send_with_transport(
-                    log_debug, send_transport, packet, packet_num, out, addr, port, v6_flow_scope
+                    log_debug,
+                    send_transport,
+                    packet,
+                    packet_num,
+                    out,
+                    addr,
+                    port,
+                    v6_flow_scope,
                 )
 
     def _close(self) -> None:
@@ -645,7 +701,7 @@ class Zeroconf(QuietLogger):
         await self.engine._async_close()  # pylint: disable=protected-access
         self._shutdown_threads()
 
-    def __enter__(self) -> 'Zeroconf':
+    def __enter__(self) -> "Zeroconf":
         return self
 
     def __exit__(  # pylint: disable=useless-return
